package com.winhc.bigdata.spark.jobs.deadbeat

import com.winhc.bigdata.spark.utils.{LoggingUtils, SparkUtils}
import org.apache.spark.internal.Logging
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @Author: XuJiakai
 * @Date: 2020/10/12 15:26
 * @Description: 失信人企业+个人 增量+全量
 */
case class dishonest_info(s: SparkSession,
                          project: String //表所在工程名
                         ) extends LoggingUtils with Logging {
  @(transient@getter) val spark: SparkSession = s

  private def is_deleted(status: Int, deleted: Int): String = if ((status + deleted) == 0) "0" else "1"

  private def calc(is_person: Boolean = true): Unit = {
    val tn = is_person match {
      case true => "company_dishonest_info_human"
      case false => "company_dishonest_info"
    }
    val cid = is_person match {
      case true => ""
      case false => ",cid"
    }

    val target_table = is_person match {
      case true => s"$project.ads_deadbeat_person"
      case false => s"$project.ads_deadbeat_company"
    }
    val inc_ads_table = s"$project.inc_ads_$tn"
    val ads_table = s"$project.ads_$tn"

    val intersect_cols = getColumns(inc_ads_table).intersect(getColumns(ads_table))
    val inc_ads_last_ds = getLastPartitionsOrElse(inc_ads_table, "0")
    val ads_last_ds = getLastPartitionsOrElse(ads_table, "0")
    spark.udf.register("is_deleted", is_deleted _)

    val view_fields =
      s"""
         |rowkey
         |${cid}
         |,name
         |,card_num
         |,pub_date as publish_date
         |,is_deleted(status,deleted) as deleted
         |""".stripMargin

    def all(): Unit = {
      sql(
        s"""
           |INSERT OVERWRITE TABLE $target_table PARTITION(ds='$inc_ads_last_ds',tn='$tn')
           |SELECT  $view_fields
           |FROM    (
           |            SELECT  *
           |                    ,ROW_NUMBER() OVER(PARTITION BY rowkey ORDER BY ds DESC ) AS num
           |            FROM    (
           |                        SELECT  ${intersect_cols.mkString(",")}
           |                        FROM    $ads_table
           |                        WHERE   ds = '$ads_last_ds'
           |                        UNION ALL
           |                        SELECT  ${intersect_cols.mkString(",")}
           |                        FROM    $inc_ads_table
           |                        WHERE   ds > '$ads_last_ds'
           |                    ) AS t1
           |        ) AS t2
           |WHERE   t2.num = 1
           |""".stripMargin)
    }

    def inc(last_ds: String): Unit = {
      val tmp_tab = s"tmp_deadbeat_$tn"
      sql(
        s"""
           |SELECT  $view_fields
           |        ,ds
           |FROM    $inc_ads_table
           |WHERE   ds > '$last_ds'
           |""".stripMargin)
        .createOrReplaceTempView(tmp_tab)

      sql(
        s"""
           |INSERT OVERWRITE TABLE $target_table PARTITION(ds='$inc_ads_last_ds',tn='$tn')
           |SELECT  rowkey
           |        ${cid}
           |        ,name
           |        ,card_num
           |        ,publish_date
           |        ,deleted
           |FROM    $tmp_tab
           |""".stripMargin)

    }

    val ds_cols = sql(s"show partitions $target_table").collect()
      .map(_.getString(0))
      .filter(_.contains(tn))
      .flatMap(_.split("/"))
      .filter(_.contains("ds"))
      .map(_.split("=")(1))
    if (ds_cols.isEmpty) {
      println("全量计算，失信人-个人")
      all()
    } else {
      val max = ds_cols.max
      println(s"增量计算：$max，失信人-个人")
      inc(max)
    }
  }

  def person(): Unit = calc()

  def company(): Unit = calc(false)

}

object dishonest_info {
  def main(args: Array[String]): Unit = {
    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> "winhc_eci_dev",
      "spark.hadoop.odps.spark.local.partition.amt" -> "1000"
    )
    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)
    val di = dishonest_info(spark, "winhc_eci_dev")
    di.person()
    di.company()

    spark.stop()
  }
}
