package com.winhc.bigdata.spark.jobs.judicial

import com.winhc.bigdata.spark.udf.{BaseFunc, CompanyMapping}
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, SparkUtils}
import org.apache.spark.sql.SparkSession

import scala.collection.mutable


object JudicialCaseRelationPre10 {
  def main(args: Array[String]): Unit = {
    val project = "winhc_eci_dev"
    println(
      s"""
         |project: $project
         |""".stripMargin)

    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> s"$project",
      "spark.hadoop.odps.spark.local.partition.amt" -> "1000"
    )
    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)
    JudicialCaseRelationPre10(spark, project).precalc()
    spark.stop()
  }
}


//司法案件-被执行人数据预处理
case class JudicialCaseRelationPre10(s: SparkSession, project: String
                                   ) extends LoggingUtils with CompanyMapping with BaseFunc {
  override protected val spark: SparkSession = s

  def precalc(): Unit = {
    prepareFunctions(spark)
    case_no_trim_udf()
    //被执行人(企业)
    var lastDsIncAds = BaseUtil.getPartion(s"$project.inc_ads_company_zxr", spark)
    spark.sparkContext.setJobDescription(s"处理zxr($lastDsIncAds)")
    sql(
      s"""
         |insert ${if (isWindows) "INTO" else "OVERWRITE"} table $project.ads_judicial_case_relation_pre partition(ds='$lastDsIncAds',tn='zxr')
         |select
         |  judicase_id
         |  ,flag
         |  ,title
         |  ,case_type
         |  ,case_reason
         |  ,case_no
         |  ,court_name
         |  ,case_stage
         |  ,yg_name
         |  ,bg_name
         |  ,date
         |  ,detail_id
         |  ,case_amt
         |from (
         |      select
         |      md5(cleanup(case_no)) as judicase_id
         |      ,"7" as flag
         |      ,concat_ws('',cname,'被执行人') as title
         |      ,concat_ws('',case_type(case_no)) as case_type
         |      ,null as case_reason
         |      ,case_no
         |      ,court as court_name
         |      ,concat_ws('',case_stage(case_no)) as case_stage
         |      ,null as yg_name
         |      ,cname as bg_name
         |      ,case_create_time as date
         |      ,rowkey as detail_id
         |      ,exec_money as case_amt
         |      ,row_number() over(partition by rowkey order by update_time desc) num
         |      from $project.inc_ads_company_zxr
         |      where length(case_no) > 0 and ds > '0'
         |   )
         |where num = 1
         |""".stripMargin).show(10, false)


    //被执行人预处理(个人）
    val columns: Seq[String] = spark.table(s"$project.inc_ads_company_zxr_person").schema.map(_.name).filter(_!="flag")
    lastDsIncAds = BaseUtil.getPartion(s"$project.inc_ads_company_zxr_person", spark)
    spark.sparkContext.setJobDescription(s"处理zxr_person($lastDsIncAds)")
    //1、先从被执行人中用name和case_no关联补全身份证号码
    sql(
      s"""
         |--先从company_zxr_person存量、增量表获取
         |SELECT rowkey
         |    ,0 AS flag
         |    ,new_cid
         |    ,cid
         |    ,id
         |    ,cname
         |    ,sex
         |    ,card
         |    ,court
         |    ,case_create_time
         |    ,A.case_no AS case_no
         |    ,gist_id
         |    ,case_state
         |    ,exec_money
         |    ,type
         |    ,source
         |    ,status
         |    ,appro_time
         |    ,A.create_time AS create_time
         |    ,A.update_time AS update_time
         |    ,A.deleted AS deleted
         |FROM(
         |    SELECT *
         |    FROM(
         |        SELECT *
         |        FROM(
         |            SELECT *
         |            ,ROW_NUMBER() OVER (PARTITION BY cleanup(CONCAT_WS('',cname,case_no)) ORDER BY update_time DESC ) num
         |            FROM(
         |                SELECT ${columns.mkString(",")}
         |                FROM $project.ads_company_zxr_person
         |                WHERE ds>'0'
         |                UNION ALL
         |                SELECT ${columns.mkString(",")}
         |                FROM $project.inc_ads_company_zxr_person
         |                WHERE ds>'0'
         |            )
         |        )
         |        WHERE num=1
         |    )
         | ) A
         |
         |""".stripMargin
    ).createOrReplaceTempView("tmp_person_cloze_1")

    //3、再从自身表中根据name和company_name去补全身份证号码
//    sql(
//      s"""
//         |--先从前面结果表获取
//         |INSERT OVERWRITE TABLE winhc_eci_dev.ads_company_zxr_person_cloze PARTITION(ds=$lastDsIncAds)
//         |SELECT A.rowkey
//         |    ,IF(A.card IS NULL AND D.card IS NOT NULL,3,0) AS flag
//         |    ,A.new_cid
//         |    ,A.cid
//         |    ,A.id
//         |    ,A.cname
//         |    ,A.sex
//         |    ,COALESCE(A.card,D.card) AS card
//         |    ,A.court
//         |    ,A.case_create_time
//         |    ,A.case_no
//         |    ,A.gist_id
//         |    ,A.case_state
//         |    ,A.exec_money
//         |    ,A.type
//         |    ,A.source
//         |    ,A.status
//         |    ,A.appro_time
//         |    ,A.create_time
//         |    ,A.update_time
//         |    ,A.deleted
//         |FROM tmp_person_cloze_1 A
//         |LEFT JOIN (tmp_person_cloze_1 D)
//         |
//         |ON A.cname=D.cname and d.card is not null  --根据实际数据情况
//         |""".stripMargin
//    )//.createOrReplaceTempView("tmp_person_cloze_3")

    sql(
      s"""
         |insert ${if (isWindows) "INTO" else "OVERWRITE"} table $project.ads_judicial_case_relation_pre partition(ds='$lastDsIncAds',tn='zxr_person')
         |select
         |  judicase_id
         |  ,flag
         |  ,title
         |  ,case_type
         |  ,case_reason
         |  ,case_no
         |  ,court
         |  ,case_stage
         |  ,yg_name
         |  ,bg_name
         |  ,date
         |  ,detail_id
         |  ,case_amt
         |from (
         |      select
         |      md5(cleanup(case_no)) as judicase_id
         |      ,"10" as flag
         |      ,concat_ws('',cname,'被执行人') AS title
         |      ,concat_ws('',case_type(case_no)) as case_type
         |      ,NULL AS case_reason
         |      ,case_no
         |      ,court
         |      ,concat_ws('',case_stage(case_no)) as case_stage
         |      ,NULL as yg_name
         |      ,cname as bg_name
         |      ,case_create_time as date
         |      ,rowkey as detail_id
         |      ,exec_money as case_amt
         |      ,row_number() over(partition by rowkey order by update_time desc) num
         |      from tmp_person_cloze_1
         |      where length(case_no) > 0
         |   )
         |where num = 1
         |""".stripMargin).show(10, false)

  }
}